@@ -1,4 +1,4 @@
-optcal<-function(z,p,exact=NULL,ncontrol=1,tol=NULL,rank=T){
+optcal<-function(z,p,exact=NULL,ncontrol=1,tol=NULL,rank=T,subX=NULL){
 
   #check input
   stopifnot(is.vector(z))
@@ -12,18 +12,30 @@ optcal<-function(z,p,exact=NULL,ncontrol=1,tol=NULL,rank=T){
     else tol<-0.01
   }
 
+  if (!is.null(subX)){
+    subX<-as.factor(subX)
+    nsubXlevels<-nlevels(subX)
+    levels(subX)<-1:nsubXlevels
+  }
+
   if (!is.null(exact)){
     exact<-as.factor(exact)
     nexactlevels<-nlevels(exact)
     levels(exact)<-1:nexactlevels
     tb<-table(z,exact)
+    if (is.null(subX)) TF<-T
+    else TF<-all(tb[2,]<=tb[1,])||(nlevels(exact)!=nlevels(subX))||any(exact!=subX)
     if (!all(tb[2,]<=tb[1,])){
-      stop("An exact match for exact is infeasible for every caliper.")
+      if (is.null(subX) || TF){
+        stop("An exact match for exact is infeasible for every caliper.")
+      }
     }
     ratio<-tb[1,]/tb[2,]
     order_ratio<-order(ratio)
     exactlevels<-(1:nexactlevels)[order_ratio]
   }
+  else TF<-T
+
 
   or<-rank(p,ties.method='min')
   #sort input
@@ -60,7 +72,7 @@ optcal<-function(z,p,exact=NULL,ncontrol=1,tol=NULL,rank=T){
       right<-left
       for (i in 1:nt){
         close<-cid[(abs(u1[i]-u0)<=caliper)]
-        if (length(close)==0) return(NULL)
+        if ((length(close)==0) & TF) return(NULL)
         left[i]<-min(close)
         right[i]<-max(close)
       }
@@ -74,7 +86,7 @@ optcal<-function(z,p,exact=NULL,ncontrol=1,tol=NULL,rank=T){
     if (!is.null(lr)){
       res<-glover(lr$left,lr$right)
     }
-    if ((!is.null(lr)) && (res==1)){
+    if ((!is.null(lr)) && (res>=min(nt,nc)/nt)){
       return(list(caliper=lowc,interval=c(lowc,lowc),interval.length=0))
     }
 
@@ -82,7 +94,7 @@ optcal<-function(z,p,exact=NULL,ncontrol=1,tol=NULL,rank=T){
     if (!is.null(lr)){
       res<-glover(lr$left,lr$right)
     }
-    if (is.null(lr)||(res==0)){
+    if (is.null(lr)||(res<min(nt,nc)/nt)){
       stop('There is no feasible matching.')
     }
 
@@ -91,10 +103,11 @@ optcal<-function(z,p,exact=NULL,ncontrol=1,tol=NULL,rank=T){
       lr<-leftright(midc)
       if (!is.null(lr)){
         res<-glover(lr$left,lr$right)}
-      if (is.null(lr)||(res==0)){
+      if (is.null(lr)||(res<min(nt,nc)/nt)){
         lowc<-midc
+      }else{
+        highc<-midc
       }
-      else highc<-midc
     }
     list(caliper=highc,interval=c(lowc,highc),interval.length=highc-lowc)
   }
@@ -106,21 +119,25 @@ optcal<-function(z,p,exact=NULL,ncontrol=1,tol=NULL,rank=T){
     optcalone(use1,use0,ub,lb,tol)
   }else{
     cals<-0
+    result<-NULL
     for (ee in 1:nexactlevels){
       ei<-exactlevels[ee]
       use1e<-use1[ex1==ei]
       use0e<-use0[ex0==ei]
       use<-c(use1e,use0e)
-      ub<-max(use)-min(use)
-      lb<-max(cals-tol,0)
-      if (ub>=lb){
-        ree<-optcalone(use1e,use0e,ub,lb,tol)
-        if (cals<ree$caliper){
-          result<-ree
-          cals<-ree$caliper
+      if (length(use)>0){
+        ub<-max(use)-min(use)
+        lb<-max(cals-tol,0)
+        if (ub>=lb){
+          ree<-optcalone(use1e,use0e,ub,lb,tol)
+          if (cals<ree$caliper){
+            result<-ree
+            cals<-ree$caliper
+          }
         }
       }
     }
-    result
+    if (is.null(result)) list(caliper=0,interval=c(0,0),interval.length=0)
+    else result
   }
 }
